"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var bt = require("@babel/types");
var getDocblock_1 = require("../utils/getDocblock");
var getDoclets_1 = require("../utils/getDoclets");
var getTypeFromAnnotation_1 = require("../utils/getTypeFromAnnotation");
var transformTagsIntoObject_1 = require("../utils/transformTagsIntoObject");
var propHandler_1 = require("./propHandler");
function propHandler(documentation, path) {
    if (bt.isClassDeclaration(path.node)) {
        path
            .get('body')
            .get('body')
            .filter(function (p) { return bt.isClassProperty(p.node) && !!p.node.decorators; })
            .forEach(function (propPath) {
            var propDeco = (propPath.get('decorators') || []).filter(function (p) {
                var exp = bt.isCallExpression(p.node.expression)
                    ? p.node.expression.callee
                    : p.node.expression;
                return bt.isIdentifier(exp) && exp.name === 'Prop';
            });
            if (!propDeco.length) {
                return;
            }
            var propName = bt.isIdentifier(propPath.node.key) ? propPath.node.key.name : undefined;
            if (!propName) {
                return;
            }
            var propDescriptor = documentation.getPropDescriptor(propName);
            // description
            var docBlock = getDocblock_1.default(propPath);
            var jsDoc = docBlock ? getDoclets_1.default(docBlock) : { description: '', tags: [] };
            var jsDocTags = jsDoc.tags ? jsDoc.tags : [];
            if (jsDocTags) {
                propDescriptor.tags = transformTagsIntoObject_1.default(jsDocTags);
            }
            if (jsDoc.description) {
                propDescriptor.description = jsDoc.description;
            }
            if (propPath.node.typeAnnotation) {
                propDescriptor.type = getTypeFromAnnotation_1.default(propPath.node.typeAnnotation);
            }
            var propDecoratorPath = propDeco[0].get('expression');
            if (bt.isCallExpression(propDecoratorPath.node)) {
                var propDecoratorArg = propDecoratorPath.get('arguments', 0);
                if (propDecoratorArg && bt.isObjectExpression(propDecoratorArg.node)) {
                    var propsPath = propDecoratorArg
                        .get('properties')
                        .filter(function (p) { return bt.isObjectProperty(p.node); });
                    // if there is no type annotation, get it from the decorators arguments
                    if (!propPath.node.typeAnnotation) {
                        propHandler_1.describeType(propsPath, propDescriptor);
                    }
                    propHandler_1.describeDefault(propsPath, propDescriptor);
                    propHandler_1.describeRequired(propsPath, propDescriptor);
                }
            }
        });
    }
}
exports.default = propHandler;
//# sourceMappingURL=classPropHandler.js.map