"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var bt = require("@babel/types");
var getDocblock_1 = require("../utils/getDocblock");
var getDoclets_1 = require("../utils/getDoclets");
var transformTagsIntoObject_1 = require("../utils/transformTagsIntoObject");
// tslint:disable-next-line:no-var-requires
var recast = require("recast");
function propHandler(documentation, path) {
    if (bt.isObjectExpression(path.node)) {
        var propsPath = path
            .get('properties')
            .filter(function (p) { return bt.isObjectProperty(p.node) && p.node.key.name === 'props'; });
        // if no prop return
        if (!propsPath.length) {
            return;
        }
        var propsValuePath = propsPath[0].get('value');
        if (bt.isObjectExpression(propsValuePath.node)) {
            var objProp = propsValuePath.get('properties');
            // filter non object properties
            var objPropFiltered = objProp.filter(function (p) { return bt.isProperty(p.node); });
            objPropFiltered.forEach(function (prop) {
                var propNode = prop.node;
                // description
                var docBlock = getDocblock_1.default(prop);
                var jsDoc = docBlock ? getDoclets_1.default(docBlock) : { description: '', tags: [] };
                var jsDocTags = jsDoc.tags ? jsDoc.tags : [];
                // if it's the v-model describe it only as such
                var propName = jsDocTags.some(function (t) { return t.title === 'model'; }) ? 'v-model' : propNode.key.name;
                var propDescriptor = documentation.getPropDescriptor(propName);
                var propValuePath = prop.get('value');
                propDescriptor.tags = jsDocTags.length ? transformTagsIntoObject_1.default(jsDocTags) : {};
                if (jsDoc.description) {
                    propDescriptor.description = jsDoc.description;
                }
                if (bt.isArrayExpression(propValuePath.node)) {
                    // I am not sure this case is valid vuejs
                    propDescriptor.type = { name: 'array' };
                }
                else if (bt.isObjectExpression(propValuePath.node)) {
                    var propPropertiesPath = propValuePath
                        .get('properties')
                        .filter(function (p) { return bt.isObjectProperty(p.node); });
                    // type
                    describeType(propPropertiesPath, propDescriptor);
                    // required
                    describeRequired(propPropertiesPath, propDescriptor);
                    // default
                    describeDefault(propPropertiesPath, propDescriptor);
                }
                else if (bt.isIdentifier(propValuePath.node)) {
                    // contents of the prop is it's type
                    propDescriptor.type = getTypeFromTypePath(propValuePath);
                }
                else {
                    propDescriptor.type = {
                        name: recast.print(prop.get('value')).code,
                        func: true,
                    };
                }
            });
        }
        else if (bt.isArrayExpression(propsValuePath.node)) {
            propsValuePath
                .get('elements')
                .filter(function (e) { return bt.isStringLiteral(e.node); })
                .forEach(function (e) {
                var propDescriptor = documentation.getPropDescriptor(e.node.value);
                propDescriptor.type = { name: 'undefined' };
                propDescriptor.required = '';
            });
        }
    }
}
exports.default = propHandler;
function describeType(propPropertiesPath, propDescriptor) {
    var typeArray = propPropertiesPath.filter(function (p) { return p.node.key.name === 'type'; });
    if (typeArray.length) {
        propDescriptor.type = getTypeFromTypePath(typeArray[0].get('value'));
    }
    else {
        // deduce the type from default expression
        var defaultArray = propPropertiesPath.filter(function (p) { return p.node.key.name === 'default'; });
        if (defaultArray.length) {
            var typeNode = defaultArray[0].node;
            var func = bt.isArrowFunctionExpression(typeNode.value) || bt.isFunctionExpression(typeNode.value);
            var typeValueNode = defaultArray[0].get('value').node;
            var typeName = typeof typeValueNode.value;
            propDescriptor.type = { name: func ? 'func' : typeName };
        }
    }
}
exports.describeType = describeType;
function getTypeFromTypePath(typePath) {
    var typeNode = typePath.node;
    var typeName = bt.isArrayExpression(typeNode)
        ? typeNode.elements
            .map(function (t) { return (t && bt.isIdentifier(t) ? t.name.toLowerCase() : 'undefined'); })
            .join('|')
        : typeNode && bt.isIdentifier(typeNode)
            ? typeNode.name.toLowerCase()
            : 'undefined';
    return {
        name: typeName === 'function' ? 'func' : typeName,
    };
}
function describeRequired(propPropertiesPath, propDescriptor) {
    var requiredArray = propPropertiesPath.filter(function (p) { return p.node.key.name === 'required'; });
    var requiredNode = requiredArray.length ? requiredArray[0].get('value').node : undefined;
    propDescriptor.required =
        requiredNode && bt.isBooleanLiteral(requiredNode) ? requiredNode.value : '';
}
exports.describeRequired = describeRequired;
function describeDefault(propPropertiesPath, propDescriptor) {
    var defaultArray = propPropertiesPath.filter(function (p) { return p.node.key.name === 'default'; });
    if (defaultArray.length) {
        var defaultPath = defaultArray[0].get('value');
        var parenthesized = false;
        if (bt.isArrowFunctionExpression(defaultPath.node) &&
            bt.isObjectExpression(defaultPath.node.body) // if () => ({})
        ) {
            defaultPath = defaultPath.get('body');
            var extra = defaultPath.node.extra;
            if (extra && extra.parenthesized) {
                parenthesized = true;
            }
        }
        var rawValue = recast.print(defaultPath).code;
        propDescriptor.defaultValue = {
            func: bt.isFunction(defaultPath.node),
            value: parenthesized ? rawValue.slice(1, rawValue.length - 1) : rawValue,
        };
    }
}
exports.describeDefault = describeDefault;
//# sourceMappingURL=propHandler.js.map