"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var bt = require("@babel/types");
// tslint:disable-next-line:no-var-requires
var recast = require("recast");
function default_1(ast, variableFilter) {
    var variables = {};
    var importedVariablePaths = {};
    // get imported variable names and filepath
    recast.visit(ast.program, {
        visitImportDeclaration: function (astPath) {
            if (!astPath.node.source) {
                return false;
            }
            var filePath = astPath.node.source.value;
            var specifiers = astPath.get('specifiers');
            specifiers.each(function (s) {
                var varName = s.node.local.name;
                var exportName = bt.isImportSpecifier(s.node) ? s.node.imported.name : 'default';
                importedVariablePaths[varName] = { filePath: filePath, exportName: exportName };
            });
            return false;
        },
    });
    recast.visit(ast.program, {
        visitExportNamedDeclaration: function (astPath) {
            var specifiers = astPath.get('specifiers');
            if (astPath.node.source) {
                var filePath_1 = astPath.node.source.value;
                specifiers.each(function (s) {
                    var varName = s.node.exported.name;
                    var exportName = s.node.local.name;
                    if (variableFilter.indexOf(varName) > -1) {
                        variables[varName] = { filePath: filePath_1, exportName: exportName };
                    }
                });
            }
            else {
                specifiers.each(function (s) {
                    var varName = s.node.exported.name;
                    var middleName = s.node.local.name;
                    var importedVar = importedVariablePaths[middleName];
                    if (importedVar && variableFilter.indexOf(varName) > -1) {
                        variables[varName] = importedVar;
                    }
                });
            }
            return false;
        },
        visitExportDefaultDeclaration: function (astPath) {
            if (variableFilter.indexOf('default') > -1) {
                var middleNameDeclaration = astPath.node.declaration;
                if (bt.isIdentifier(middleNameDeclaration)) {
                    var middleName = middleNameDeclaration.name;
                    var importedVar = importedVariablePaths[middleName];
                    if (importedVar) {
                        variables.default = importedVar;
                    }
                }
            }
            return false;
        },
    });
    return variables;
}
exports.default = default_1;
//# sourceMappingURL=resolveImmediatelyExported.js.map